Package: MLmetrics
Type: Package
Title: Machine Learning Evaluation Metrics
Version: 1.1.3
Authors@R: person("Yachen", "Yan", email = "yanyachen21@gmail.com", role = c("aut", "cre"))
Description: A collection of evaluation metrics, including loss, score and
    utility functions, that measure regression, classification and ranking performance.
URL: https://github.com/yanyachen/MLmetrics
BugReports: https://github.com/yanyachen/MLmetrics/issues
Depends: R (>= 2.10)
Imports: stats, utils, ROCR
Suggests: e1071
License: GPL-2
RoxygenNote: 5.0.1
NeedsCompilation: no
Packaged: 2024-04-01 02:06:00 UTC; Administrator
Author: Yachen Yan [aut, cre]
Maintainer: Yachen Yan <yanyachen21@gmail.com>
Repository: CRAN
Date/Publication: 2024-04-13 23:50:05 UTC
Built: R 4.6.0; ; 2026-04-26 02:31:33 UTC; unix

# Generated by roxygen2: do not edit by hand

export(AUC)
export(Accuracy)
export(Area_Under_Curve)
export(ConfusionDF)
export(ConfusionMatrix)
export(F1_Score)
export(FBeta_Score)
export(GainAUC)
export(Gini)
export(KS_Stat)
export(LiftAUC)
export(LogLoss)
export(MAE)
export(MAPE)
export(MSE)
export(MedianAE)
export(MedianAPE)
export(MultiLogLoss)
export(NormalizedGini)
export(PRAUC)
export(Poisson_LogLoss)
export(Precision)
export(R2_Score)
export(RAE)
export(RMSE)
export(RMSLE)
export(RMSPE)
export(RRSE)
export(Recall)
export(Sensitivity)
export(Specificity)
export(ZeroOneLoss)
importFrom(stats,integrate)
importFrom(stats,median)
importFrom(stats,model.matrix)
importFrom(stats,na.omit)
importFrom(stats,splinefun)

#' MLmetrics: Machine Learning Evaluation Metrics
#'
#' A collection of evaluation metrics, including loss, score and utility functions, 
#' that measure regression and classification performance.
#'
#' @docType package
#' @name MLmetrics
NULL

#' @title Calculate the Area Under the Curve
#'
#' @description
#' Calculate the area under the curve.
#'
#' @param x the x-points of the curve
#' @param y the y-points of the curve
#' @param method can be "trapezoid" (default), "step" or "spline"
#' @param na.rm a logical value indicating whether NA values should be stripped before the computation proceeds
#' @return Area Under the Curve (AUC)
#' @examples
#' x <- seq(0, pi, length.out = 200)
#' plot(x = x, y = sin(x), type = "l")
#' Area_Under_Curve(x = x, y = sin(x), method = "trapezoid", na.rm = TRUE)
#' @importFrom stats integrate
#' @importFrom stats na.omit
#' @importFrom stats splinefun
#' @export

Area_Under_Curve <- function(x, y, method = c("trapezoid", "step", "spline"), na.rm = FALSE) {
  if (na.rm == TRUE) {
    xy_cbind <- na.omit(cbind(x, y))
    x <- xy_cbind[, 1]
    y <- xy_cbind[, 2]
  }
  if (length(x) != length(y)) {
    stop("length x must equal length y")
  }
  idx <- order(x)
  x <- x[idx]
  y <- y[idx]
  switch(match.arg(arg = method, choices = c("trapezoid", "step","spline")),
         trapezoid = {
           AUC <- sum((apply(cbind(y[-length(y)], y[-1]), 1, mean)) *(x[-1] - x[-length(x)]))},
         step = {
           AUC <- sum(y[-length(y)] * (x[-1] - x[-length(x)]))},
         spline = {
           AUC <- integrate(splinefun(x, y, method = "natural"), lower = min(x),upper = max(x))$value})
  return(AUC)
}

#' @title Normalized Zero-One Loss (Classification Error Loss)
#'
#' @description
#' Compute the normalized zero-one classification loss.
#'
#' @param y_pred Predicted labels vector, as returned by a classifier
#' @param y_true Ground truth (correct) 0-1 labels vector
#' @return Zero-One Loss
#' @examples
#' data(cars)
#' logreg <- glm(formula = vs ~ hp + wt,
#'               family = binomial(link = "logit"), data = mtcars)
#' pred <- ifelse(logreg$fitted.values < 0.5, 0, 1)
#' ZeroOneLoss(y_pred = pred, y_true = mtcars$vs)
#' @export

ZeroOneLoss <- function(y_pred, y_true) {
  ZeroOneLoss <- mean(y_true != y_pred)
  return(ZeroOneLoss)
}


#' @title Accuracy
#'
#' @description
#' Compute the accuracy classification score.
#'
#' @param y_pred Predicted labels vector, as returned by a classifier
#' @param y_true Ground truth (correct) 0-1 labels vector
#' @return Accuracy
#' @examples
#' data(cars)
#' logreg <- glm(formula = vs ~ hp + wt,
#'               family = binomial(link = "logit"), data = mtcars)
#' pred <- ifelse(logreg$fitted.values < 0.5, 0, 1)
#' Accuracy(y_pred = pred, y_true = mtcars$vs)
#' @export

Accuracy <- function(y_pred, y_true) {
  Accuracy <- mean(y_true == y_pred)
  return(Accuracy)
}


#' @title Confusion Matrix
#'
#' @description
#' Compute confusion matrix to evaluate the accuracy of a classification.
#'
#' @param y_pred Predicted labels vector, as returned by a classifier
#' @param y_true Ground truth (correct) 0-1 labels vector
#' @return a table of Confusion Matrix
#' @examples
#' data(cars)
#' logreg <- glm(formula = vs ~ hp + wt,
#'               family = binomial(link = "logit"), data = mtcars)
#' pred <- ifelse(logreg$fitted.values < 0.5, 0, 1)
#' ConfusionMatrix(y_pred = pred, y_true = mtcars$vs)
#' @export

ConfusionMatrix <- function(y_pred, y_true) {
  Confusion_Mat <- table(y_true, y_pred)
  return(Confusion_Mat)
}


#' @title Confusion Matrix (Data Frame Format)
#'
#' @description
#' Compute data frame format confusion matrix for internal usage.
#'
#' @param y_pred Predicted labels vector, as returned by a classifier
#' @param y_true Ground truth (correct) 0-1 labels vector
#' @return a data.frame of Confusion Matrix
#' @examples
#' data(cars)
#' logreg <- glm(formula = vs ~ hp + wt,
#'               family = binomial(link = "logit"), data = mtcars)
#' pred <- ifelse(logreg$fitted.values < 0.5, 0, 1)
#' ConfusionDF(y_pred = pred, y_true = mtcars$vs)
#' @keywords internal
#' @export

ConfusionDF <- function(y_pred, y_true) {
  Confusion_DF <- transform(as.data.frame(ConfusionMatrix(y_pred, y_true)),
                            y_true = as.character(y_true),
                            y_pred = as.character(y_pred),
                            Freq = as.integer(Freq))
  return(Confusion_DF)
}
utils::globalVariables("Freq")

#' @title Precision
#'
#' @description
#' Compute the precision score.
#'
#' @param y_pred Predicted labels vector, as returned by a classifier
#' @param y_true Ground truth (correct) 0-1 labels vector
#' @param positive An optional character string for the factor level that
#'   corresponds to a "positive" result
#' @return Precision
#' @examples
#' data(cars)
#' logreg <- glm(formula = vs ~ hp + wt,
#'               family = binomial(link = "logit"), data = mtcars)
#' pred <- ifelse(logreg$fitted.values < 0.5, 0, 1)
#' Precision(y_pred = pred, y_true = mtcars$vs, positive = "0")
#' Precision(y_pred = pred, y_true = mtcars$vs, positive = "1")
#' @export

Precision <- function(y_true, y_pred, positive = NULL) {
  Confusion_DF <- ConfusionDF(y_pred, y_true)
  if (is.null(positive) == TRUE) positive <- as.character(Confusion_DF[1,1])
  TP <- as.integer(subset(Confusion_DF, y_true==positive & y_pred==positive)["Freq"])
  FP <- as.integer(sum(subset(Confusion_DF, y_true!=positive & y_pred==positive)["Freq"]))
  Precision <- TP/(TP+FP)
  return(Precision)
}


#' @title Recall
#'
#' @description
#' Compute the recall score.
#'
#' @param y_pred Predicted labels vector, as returned by a classifier
#' @param y_true Ground truth (correct) 0-1 labels vector
#' @param positive An optional character string for the factor level that
#'   corresponds to a "positive" result
#' @return Recall
#' @examples
#' data(cars)
#' logreg <- glm(formula = vs ~ hp + wt,
#'               family = binomial(link = "logit"), data = mtcars)
#' pred <- ifelse(logreg$fitted.values < 0.5, 0, 1)
#' Recall(y_pred = pred, y_true = mtcars$vs, positive = "0")
#' Recall(y_pred = pred, y_true = mtcars$vs, positive = "1")
#' @export

Recall <- function(y_true, y_pred, positive = NULL) {
  Confusion_DF <- ConfusionDF(y_pred, y_true)
  if (is.null(positive) == TRUE) positive <- as.character(Confusion_DF[1,1])
  TP <- as.integer(subset(Confusion_DF, y_true==positive & y_pred==positive)["Freq"])
  FN <- as.integer(sum(subset(Confusion_DF, y_true==positive & y_pred!=positive)["Freq"]))
  Recall <- TP/(TP+FN)
  return(Recall)
}


#' @title Sensitivity
#'
#' @description
#' Compute the sensitivity score.
#'
#' @param y_pred Predicted labels vector, as returned by a classifier
#' @param y_true Ground truth (correct) 0-1 labels vector
#' @param positive An optional character string for the factor level that
#'   corresponds to a "positive" result
#' @return Sensitivity
#' @examples
#' data(cars)
#' logreg <- glm(formula = vs ~ hp + wt,
#'               family = binomial(link = "logit"), data = mtcars)
#' pred <- ifelse(logreg$fitted.values < 0.5, 0, 1)
#' Sensitivity(y_pred = pred, y_true = mtcars$vs, positive = "0")
#' Sensitivity(y_pred = pred, y_true = mtcars$vs, positive = "1")
#' @export

Sensitivity  <- function(y_true, y_pred, positive = NULL) {
  Confusion_DF <- ConfusionDF(y_pred, y_true)
  if (is.null(positive) == TRUE) positive <- as.character(Confusion_DF[1,1])
  TP <- as.integer(subset(Confusion_DF, y_true==positive & y_pred==positive)["Freq"])
  FN <- as.integer(sum(subset(Confusion_DF, y_true==positive & y_pred!=positive)["Freq"]))
  Sensitivity <- TP/(TP+FN)
  return(Sensitivity)
}


#' @title Specificity
#'
#' @description
#' Compute the specificity score.
#'
#' @param y_pred Predicted labels vector, as returned by a classifier
#' @param y_true Ground truth (correct) 0-1 labels vector
#' @param positive An optional character string for the factor level that
#'   corresponds to a "positive" result
#' @return Specificity
#' @examples
#' data(cars)
#' logreg <- glm(formula = vs ~ hp + wt,
#'               family = binomial(link = "logit"), data = mtcars)
#' pred <- ifelse(logreg$fitted.values < 0.5, 0, 1)
#' Specificity(y_pred = pred, y_true = mtcars$vs, positive = "0")
#' Specificity(y_pred = pred, y_true = mtcars$vs, positive = "1")
#' @export

Specificity  <- function(y_true, y_pred, positive = NULL) {
  Confusion_DF <- ConfusionDF(y_pred, y_true)
  if (is.null(positive) == TRUE) positive <- as.character(Confusion_DF[1,1])
  TN <- as.integer(subset(Confusion_DF, y_true!=positive & y_pred!=positive)["Freq"])
  FP <- as.integer(sum(subset(Confusion_DF, y_true!=positive & y_pred==positive)["Freq"]))
  Specificity <- TN/(TN+FP)
  return(Specificity)
}


#' @title F1 Score
#'
#' @description
#' Compute the F1 Score.
#'
#' @param y_pred Predicted labels vector, as returned by a classifier
#' @param y_true Ground truth (correct) 0-1 labels vector
#' @param positive An optional character string for the factor level that
#'   corresponds to a "positive" result
#' @return F1 Score
#' @examples
#' data(cars)
#' logreg <- glm(formula = vs ~ hp + wt,
#'               family = binomial(link = "logit"), data = mtcars)
#' pred <- ifelse(logreg$fitted.values < 0.5, 0, 1)
#' F1_Score(y_pred = pred, y_true = mtcars$vs, positive = "0")
#' F1_Score(y_pred = pred, y_true = mtcars$vs, positive = "1")
#' @export

F1_Score <- function(y_true, y_pred, positive = NULL) {
  Confusion_DF <- ConfusionDF(y_pred, y_true)
  if (is.null(positive) == TRUE) positive <- as.character(Confusion_DF[1,1])
  Precision <- Precision(y_true, y_pred, positive)
  Recall <- Recall(y_true, y_pred, positive)
  F1_Score <- 2 * (Precision * Recall) / (Precision + Recall)
  return(F1_Score)
}


#' @title F-Beta Score
#'
#' @description
#' Compute the F-Beta Score
#'
#' @param y_pred Predicted labels vector, as returned by a classifier
#' @param y_true Ground truth (correct) 0-1 labels vector
#' @param positive An optional character string for the factor level that
#'   corresponds to a "positive" result
#' @param beta Weight of precision in harmonic mean
#' @return F-Beta Score
#' @examples
#' data(cars)
#' logreg <- glm(formula = vs ~ hp + wt,
#'               family = binomial(link = "logit"), data = mtcars)
#' pred <- ifelse(logreg$fitted.values < 0.5, 0, 1)
#' FBeta_Score(y_pred = pred, y_true = mtcars$vs, positive = "0", beta = 2)
#' FBeta_Score(y_pred = pred, y_true = mtcars$vs, positive = "1", beta = 2)
#' @export

FBeta_Score <- function(y_true, y_pred, positive = NULL, beta = 1) {
  Confusion_DF <- ConfusionDF(y_pred, y_true)
  if (is.null(positive) == TRUE) positive <- as.character(Confusion_DF[1,1])
  Precision <- Precision(y_true, y_pred, positive)
  Recall <- Recall(y_true, y_pred, positive)
  Fbeta_Score <- (1 + beta^2) * (Precision * Recall) / (beta^2 * Precision + Recall)
  return(Fbeta_Score)
}


#' @title Log loss / Cross-Entropy Loss
#'
#' @description
#' Compute the log loss/cross-entropy loss.
#'
#' @param y_pred Predicted probabilities vector, as returned by a classifier
#' @param y_true Ground truth (correct) 0-1 labels vector
#' @return Log loss/Cross-Entropy Loss
#' @examples
#' data(cars)
#' logreg <- glm(formula = vs ~ hp + wt,
#'               family = binomial(link = "logit"), data = mtcars)
#' LogLoss(y_pred = logreg$fitted.values, y_true = mtcars$vs)
#' @export

LogLoss <- function(y_pred, y_true) {
  eps <- 1e-15
  y_pred <- pmax(pmin(y_pred, 1 - eps), eps)
  LogLoss <- -mean(y_true * log(y_pred) + (1 - y_true) * log(1 - y_pred))
  return(LogLoss)
}


#' @title Multi Class Log Loss
#'
#' @description
#' Compute the multi class log loss.
#'
#' @param y_true Ground truth (correct) labels vector or a matrix of
#'   correct labels indicating by 0-1, same format as probabilities matrix
#' @param y_pred Predicted probabilities matrix, as returned by a classifier
#' @return Multi Class Log Loss
#' @examples
#' data(iris)
#' svm.model <- e1071::svm(Species~., data = iris, probability = TRUE)
#' pred <- predict(svm.model, iris, probability = TRUE)
#' MultiLogLoss(y_true = iris$Species, y_pred = attr(pred, "probabilities"))
#' @importFrom stats model.matrix
#' @export

MultiLogLoss <- function(y_pred, y_true) {
  if (is.matrix(y_true) == FALSE) {
    y_true <- model.matrix(~ 0 + ., data.frame(as.character(y_true)))
  }
  eps <- 1e-15
  N <- nrow(y_pred)
  y_pred <- pmax(pmin(y_pred, 1 - eps), eps)
  MultiLogLoss <- (-1 / N) * sum(y_true * log(y_pred))
  return(MultiLogLoss)
}


#' @title Area Under the Receiver Operating Characteristic Curve (ROC AUC)
#'
#' @description
#' Compute the Area Under the Receiver Operating Characteristic Curve (ROC AUC) from prediction scores.
#'
#' @param y_pred Predicted probabilities vector, as returned by a classifier
#' @param y_true Ground truth (correct) 0-1 labels vector
#' @return Area Under the ROC Curve (ROC AUC)
#' @examples
#' data(cars)
#' logreg <- glm(formula = vs ~ hp + wt,
#'               family = binomial(link = "logit"), data = mtcars)
#' AUC(y_pred = logreg$fitted.values, y_true = mtcars$vs)
#' @export

AUC <- function(y_pred, y_true) {
  rank <- rank(y_pred)
  n_pos <- as.double(sum(y_true == 1))
  n_neg <- as.double(sum(y_true == 0))
  AUC <- (sum(rank[y_true == 1]) - n_pos * (n_pos + 1) / 2) / (n_pos * n_neg)
  return(AUC)
}


#' @title Gini Coefficient
#'
#' @description
#' Compute the Gini Coefficient.
#'
#' @param y_pred Predicted probabilities vector, as returned by a classifier
#' @param y_true Ground truth (correct) 0-1 labels vector
#' @return Gini Coefficient
#' @examples
#' data(cars)
#' logreg <- glm(formula = vs ~ hp + wt,
#'               family = binomial(link = "logit"), data = mtcars)
#' Gini(y_pred = logreg$fitted.values, y_true = mtcars$vs)
#' @export

Gini <- function(y_pred, y_true) {
  NormalizedGini(y_pred, y_true)
}


#' @title Area Under the Precision-Recall Curve (PR AUC)
#'
#' @description
#' Compute the Area Under the Precision-Recall Curve (PR AUC) from prediction scores.
#'
#' @param y_pred Predicted probabilities vector, as returned by a classifier
#' @param y_true Ground truth (correct) 0-1 labels vector
#' @return Area Under the PR Curve (PR AUC)
#' @examples
#' data(cars)
#' logreg <- glm(formula = vs ~ hp + wt,
#'               family = binomial(link = "logit"), data = mtcars)
#' PRAUC(y_pred = logreg$fitted.values, y_true = mtcars$vs)
#' @export

PRAUC <- function(y_pred, y_true) {
  pred_obj <- ROCR::prediction(y_pred, y_true)
  perf_obj <- ROCR::performance(pred_obj, measure = "prec", x.measure = "rec")
  PRAUC <- Area_Under_Curve(perf_obj@x.values[[1]], perf_obj@y.values[[1]], method = "trapezoid", na.rm = TRUE)
  return(PRAUC)
}


#' @title Area Under the Lift Chart
#'
#' @description
#' Compute the Area Under the Lift Chart from prediction scores.
#'
#' @param y_pred Predicted probabilities vector, as returned by a classifier
#' @param y_true Ground truth (correct) 0-1 labels vector
#' @return Area Under the Lift Chart
#' @examples
#' data(cars)
#' logreg <- glm(formula = vs ~ hp + wt,
#'               family = binomial(link = "logit"), data = mtcars)
#' LiftAUC(y_pred = logreg$fitted.values, y_true = mtcars$vs)
#' @export

LiftAUC <- function(y_pred, y_true) {
  pred_obj <- ROCR::prediction(y_pred, y_true)
  perf_obj <- ROCR::performance(pred_obj, measure = "lift", x.measure = "rpp")
  LiftAUC <- Area_Under_Curve(perf_obj@x.values[[1]], perf_obj@y.values[[1]], method = "trapezoid", na.rm = TRUE)
  return(LiftAUC)
}


#' @title Area Under the Gain Chart
#'
#' @description
#' Compute the Area Under the Gain Chart from prediction scores.
#'
#' @param y_pred Predicted probabilities vector, as returned by a classifier
#' @param y_true Ground truth (correct) 0-1 labels vector
#' @return Area Under the Gain Chart
#' @examples
#' data(cars)
#' logreg <- glm(formula = vs ~ hp + wt,
#'               family = binomial(link = "logit"), data = mtcars)
#' GainAUC(y_pred = logreg$fitted.values, y_true = mtcars$vs)
#' @export

GainAUC <- function(y_pred, y_true) {
  pred_obj <- ROCR::prediction(y_pred, y_true)
  perf_obj <- ROCR::performance(pred_obj, measure = "tpr", x.measure = "rpp")
  GainAUC <- Area_Under_Curve(perf_obj@x.values[[1]], perf_obj@y.values[[1]], method = "trapezoid", na.rm = TRUE)
  return(GainAUC)
}


#' @title Kolmogorov-Smirnov Statistic
#'
#' @description
#' Compute the Kolmogorov-Smirnov statistic.
#'
#' @param y_pred Predicted probabilities vector, as returned by a classifier
#' @param y_true Ground truth (correct) 0-1 labels vector
#' @return Kolmogorov-Smirnov statistic
#' @examples
#' data(cars)
#' logreg <- glm(formula = vs ~ hp + wt,
#'               family = binomial(link = "logit"), data = mtcars)
#' KS_Stat(y_pred = logreg$fitted.values, y_true = mtcars$vs)
#' @export

KS_Stat <- function(y_pred, y_true) {
  pred_obj <- ROCR::prediction(y_pred, y_true)
  perf_obj <- ROCR::performance(pred_obj, measure = "tpr", x.measure = "fpr")
  KS_Stat <- max(perf_obj@y.values[[1]] - perf_obj@x.values[[1]])
  return(KS_Stat)
}

#' @title Mean Square Error Loss
#'
#' @description
#' Compute the mean squared error regression loss.
#'
#' @param y_pred Estimated target values vector
#' @param y_true Ground truth (correct) target values vector
#' @return Mean Square Error Loss
#' @examples
#' data(cars)
#' reg <- lm(log(dist) ~ log(speed), data = cars)
#' MSE(y_pred = exp(reg$fitted.values), y_true = cars$dist)
#' @export

MSE <- function(y_pred, y_true) {
  MSE <- mean((y_true - y_pred)^2)
  return(MSE)
}


#' @title Root Mean Square Error Loss
#'
#' @description
#' Compute the root mean squared error regression loss.
#'
#' @param y_pred Estimated target values vector
#' @param y_true Ground truth (correct) target values vector
#' @return Root Mean Square Error Loss
#' @examples
#' data(cars)
#' reg <- lm(log(dist) ~ log(speed), data = cars)
#' RMSE(y_pred = exp(reg$fitted.values), y_true = cars$dist)
#' @export

RMSE <- function(y_pred, y_true) {
  RMSE <- sqrt(mean((y_true - y_pred)^2))
  return(RMSE)
}


#' @title Root Mean Squared Logarithmic Error Loss
#'
#' @description
#' Compute the root mean squared logarithmic error regression loss.
#'
#' @param y_pred Estimated target values vector
#' @param y_true Ground truth (correct) target values vector
#' @return Root Mean Squared Logarithmic Error Loss
#' @examples
#' data(cars)
#' reg <- lm(log(dist) ~ log(speed), data = cars)
#' RMSLE(y_pred = exp(reg$fitted.values), y_true = cars$dist)
#' @export

RMSLE <- function(y_pred, y_true) {
  RMSLE <- sqrt(mean((log(1 + y_true) - log(1 + y_pred))^2))
  return(RMSLE)
}


#' @title Root Mean Square Percentage Error Loss
#'
#' @description
#' Compute the root mean squared percentage error regression loss.
#'
#' @param y_pred Estimated target values vector
#' @param y_true Ground truth (correct) target values vector
#' @return Root Mean Squared Percentage Error Loss
#' @examples
#' data(cars)
#' reg <- lm(log(dist) ~ log(speed), data = cars)
#' RMSPE(y_pred = exp(reg$fitted.values), y_true = cars$dist)
#' @export

RMSPE <- function(y_pred, y_true) {
  RMSPE <- sqrt(mean(((y_true - y_pred) / y_true)^2))
  return(RMSPE)
}


#' @title Root Relative Squared Error Loss
#'
#' @description
#' Compute the root relative squared error regression loss.
#'
#' @param y_pred Estimated target values vector
#' @param y_true Ground truth (correct) target values vector
#' @return Root Relative Squared Error Loss
#' @examples
#' data(cars)
#' reg <- lm(log(dist) ~ log(speed), data = cars)
#' RRSE(y_pred = exp(reg$fitted.values), y_true = cars$dist)
#' @export

RRSE <- function(y_pred, y_true) {
  RRSE <- sqrt(sum((y_true - y_pred)^2) / sum((y_true - mean(y_true))^2))
  return(RRSE)
}


#' @title Mean Absolute Error Loss
#'
#' @description
#' Compute the mean absolute error regression loss.
#'
#' @param y_pred Estimated target values vector
#' @param y_true Ground truth (correct) target values vector
#' @return Mean Absolute Error Loss
#' @examples
#' data(cars)
#' reg <- lm(log(dist) ~ log(speed), data = cars)
#' MAE(y_pred = exp(reg$fitted.values), y_true = cars$dist)
#' @export

MAE <- function(y_pred, y_true) {
  MAE <- mean(abs(y_true - y_pred))
  return(MAE)
}


#' @title Mean Absolute Percentage Error Loss
#'
#' @description
#' Compute the mean absolute percentage error regression loss.
#'
#' @param y_pred Estimated target values vector
#' @param y_true Ground truth (correct) target values vector
#' @return Mean Absolute Percentage Error Loss
#' @examples
#' data(cars)
#' reg <- lm(log(dist) ~ log(speed), data = cars)
#' MAPE(y_pred = exp(reg$fitted.values), y_true = cars$dist)
#' @export

MAPE <- function(y_pred, y_true) {
  MAPE <- mean(abs((y_true - y_pred) / y_true))
  return(MAPE)
}


#' @title Median Absolute Error Loss
#'
#' @description
#' Compute the median absolute error regression loss.
#'
#' @param y_pred Estimated target values vector
#' @param y_true Ground truth (correct) target values vector
#' @return Median Absolute Error Loss
#' @examples
#' data(cars)
#' reg <- lm(log(dist) ~ log(speed), data = cars)
#' MedianAE(y_pred = exp(reg$fitted.values), y_true = cars$dist)
#' @importFrom stats median
#' @export

MedianAE <- function(y_pred, y_true) {
  MedianAE <- median(abs(y_true - y_pred))
  return(MedianAE)
}


#' @title Median Absolute Percentage Error Loss
#'
#' @description
#' Compute the Median absolute percentage error regression loss.
#'
#' @param y_pred Estimated target values vector
#' @param y_true Ground truth (correct) target values vector
#' @return Median Absolute Percentage Error Loss
#' @examples
#' data(cars)
#' reg <- lm(log(dist) ~ log(speed), data = cars)
#' MedianAPE(y_pred = exp(reg$fitted.values), y_true = cars$dist)
#' @importFrom stats median
#' @export

MedianAPE <- function(y_pred, y_true) {
  MedianAPE <- median(abs((y_true - y_pred) / y_true))
  return(MedianAPE)
}


#' @title Relative Absolute Error Loss
#'
#' @description
#' Compute the relative absolute error regression loss.
#'
#' @param y_pred Estimated target values vector
#' @param y_true Ground truth (correct) target values vector
#' @return Relative Absolute Error Loss
#' @examples
#' data(cars)
#' reg <- lm(log(dist) ~ log(speed), data = cars)
#' RAE(y_pred = exp(reg$fitted.values), y_true = cars$dist)
#' @export

RAE <- function(y_pred, y_true) {
  RAE <- sum(abs(y_true - y_pred)) / sum(abs(y_true - mean(y_true)))
  return(RAE)
}


#' @title R-Squared (Coefficient of Determination) Regression Score
#'
#' @description
#' Compute the R-Squared (Coefficient of Determination) Regression Score.
#'
#' @param y_pred Estimated target values vector
#' @param y_true Ground truth (correct) target values vector
#' @return R^2 Score
#' @examples
#' data(cars)
#' reg <- lm(log(dist) ~ log(speed), data = cars)
#' R2_Score(y_pred = exp(reg$fitted.values), y_true = cars$dist)
#' @export

R2_Score <- function(y_pred, y_true) {
  R2_Score <- 1 - sum((y_true - y_pred)^2) / sum((y_true - mean(y_true))^2)
  return(R2_Score)
}

#' @title Poisson Log loss
#'
#' @description
#' Compute the log loss/cross-entropy loss.
#'
#' @param y_pred Predicted labels vector, as returned by a model
#' @param y_true Ground truth (correct) labels vector
#' @return Log loss/Cross-Entropy Loss
#' @examples
#' d_AD <- data.frame(treatment = gl(3,3), outcome = gl(3,1,9),
#'                    counts = c(18,17,15,20,10,20,25,13,12))
#' glm_poisson <- glm(counts ~ outcome + treatment,
#'                    family = poisson(link = "log"), data = d_AD)
#' Poisson_LogLoss(y_pred = glm_poisson$fitted.values, y_true = d_AD$counts)
#' @export

Poisson_LogLoss <- function(y_pred, y_true) {
  eps <- 1e-15
  y_pred <- pmax(y_pred, eps)
  Poisson_LogLoss <- mean(log(gamma(y_true + 1)) + y_pred - log(y_pred) * y_true)
  # Poisson_LogLoss <- mean(-dpois(y_true, y_pred, log = TRUE))
  return(Poisson_LogLoss)
}


#' @title Normalized Gini Coefficient
#'
#' @description
#' Compute the Normalized Gini Coefficient.
#'
#' @param y_pred Predicted labels vector, as returned by a model
#' @param y_true Ground truth (correct) labels vector
#' @return Normalized Gini Coefficient
#' @examples
#' d_AD <- data.frame(treatment = gl(3,3), outcome = gl(3,1,9),
#'                    counts = c(18,17,15,20,10,20,25,13,12))
#' glm_poisson <- glm(counts ~ outcome + treatment,
#'                    family = poisson(link = "log"), data = d_AD)
#' NormalizedGini(y_pred = glm_poisson$fitted.values, y_true = d_AD$counts)
#' @export

NormalizedGini <- function(y_pred, y_true) {
  SumGini <- function(y_pred, y_true) {
    y_true_sort <- y_true[order(y_pred, decreasing = TRUE)]
    y_random <- 1:length(y_pred) / length(y_pred)
    y_Lorentz <- cumsum(y_true_sort) / sum(y_true_sort)
    SumGini <- sum(y_Lorentz - y_random)
    return(SumGini)
  }
  NormalizedGini <- SumGini(y_pred, y_true) / SumGini(y_true, y_true)
  return(NormalizedGini)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/Classification.R
\name{MultiLogLoss}
\alias{MultiLogLoss}
\title{Multi Class Log Loss}
\usage{
MultiLogLoss(y_pred, y_true)
}
\arguments{
\item{y_pred}{Predicted probabilities matrix, as returned by a classifier}

\item{y_true}{Ground truth (correct) labels vector or a matrix of
correct labels indicating by 0-1, same format as probabilities matrix}
}
\value{
Multi Class Log Loss
}
\description{
Compute the multi class log loss.
}
\examples{
data(iris)
svm.model <- e1071::svm(Species~., data = iris, probability = TRUE)
pred <- predict(svm.model, iris, probability = TRUE)
MultiLogLoss(y_true = iris$Species, y_pred = attr(pred, "probabilities"))
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/Regression.R
\name{RMSPE}
\alias{RMSPE}
\title{Root Mean Square Percentage Error Loss}
\usage{
RMSPE(y_pred, y_true)
}
\arguments{
\item{y_pred}{Estimated target values vector}

\item{y_true}{Ground truth (correct) target values vector}
}
\value{
Root Mean Squared Percentage Error Loss
}
\description{
Compute the root mean squared percentage error regression loss.
}
\examples{
data(cars)
reg <- lm(log(dist) ~ log(speed), data = cars)
RMSPE(y_pred = exp(reg$fitted.values), y_true = cars$dist)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/Count.R
\name{NormalizedGini}
\alias{NormalizedGini}
\title{Normalized Gini Coefficient}
\usage{
NormalizedGini(y_pred, y_true)
}
\arguments{
\item{y_pred}{Predicted labels vector, as returned by a model}

\item{y_true}{Ground truth (correct) labels vector}
}
\value{
Normalized Gini Coefficient
}
\description{
Compute the Normalized Gini Coefficient.
}
\examples{
d_AD <- data.frame(treatment = gl(3,3), outcome = gl(3,1,9),
                   counts = c(18,17,15,20,10,20,25,13,12))
glm_poisson <- glm(counts ~ outcome + treatment,
                   family = poisson(link = "log"), data = d_AD)
NormalizedGini(y_pred = glm_poisson$fitted.values, y_true = d_AD$counts)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/Classification.R
\name{ZeroOneLoss}
\alias{ZeroOneLoss}
\title{Normalized Zero-One Loss (Classification Error Loss)}
\usage{
ZeroOneLoss(y_pred, y_true)
}
\arguments{
\item{y_pred}{Predicted labels vector, as returned by a classifier}

\item{y_true}{Ground truth (correct) 0-1 labels vector}
}
\value{
Zero-One Loss
}
\description{
Compute the normalized zero-one classification loss.
}
\examples{
data(cars)
logreg <- glm(formula = vs ~ hp + wt,
              family = binomial(link = "logit"), data = mtcars)
pred <- ifelse(logreg$fitted.values < 0.5, 0, 1)
ZeroOneLoss(y_pred = pred, y_true = mtcars$vs)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/Classification.R
\name{Gini}
\alias{Gini}
\title{Gini Coefficient}
\usage{
Gini(y_pred, y_true)
}
\arguments{
\item{y_pred}{Predicted probabilities vector, as returned by a classifier}

\item{y_true}{Ground truth (correct) 0-1 labels vector}
}
\value{
Gini Coefficient
}
\description{
Compute the Gini Coefficient.
}
\examples{
data(cars)
logreg <- glm(formula = vs ~ hp + wt,
              family = binomial(link = "logit"), data = mtcars)
Gini(y_pred = logreg$fitted.values, y_true = mtcars$vs)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/Utils.R
\name{Area_Under_Curve}
\alias{Area_Under_Curve}
\title{Calculate the Area Under the Curve}
\usage{
Area_Under_Curve(x, y, method = c("trapezoid", "step", "spline"),
  na.rm = FALSE)
}
\arguments{
\item{x}{the x-points of the curve}

\item{y}{the y-points of the curve}

\item{method}{can be "trapezoid" (default), "step" or "spline"}

\item{na.rm}{a logical value indicating whether NA values should be stripped before the computation proceeds}
}
\value{
Area Under the Curve (AUC)
}
\description{
Calculate the area under the curve.
}
\examples{
x <- seq(0, pi, length.out = 200)
plot(x = x, y = sin(x), type = "l")
Area_Under_Curve(x = x, y = sin(x), method = "trapezoid", na.rm = TRUE)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/MLmetrics.R
\docType{package}
\name{MLmetrics}
\alias{MLmetrics}
\alias{MLmetrics-package}
\title{MLmetrics: Machine Learning Evaluation Metrics}
\description{
A collection of evaluation metrics, including loss, score and utility functions, 
that measure regression and classification performance.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/Classification.R
\name{Sensitivity}
\alias{Sensitivity}
\title{Sensitivity}
\usage{
Sensitivity(y_true, y_pred, positive = NULL)
}
\arguments{
\item{y_true}{Ground truth (correct) 0-1 labels vector}

\item{y_pred}{Predicted labels vector, as returned by a classifier}

\item{positive}{An optional character string for the factor level that
corresponds to a "positive" result}
}
\value{
Sensitivity
}
\description{
Compute the sensitivity score.
}
\examples{
data(cars)
logreg <- glm(formula = vs ~ hp + wt,
              family = binomial(link = "logit"), data = mtcars)
pred <- ifelse(logreg$fitted.values < 0.5, 0, 1)
Sensitivity(y_pred = pred, y_true = mtcars$vs, positive = "0")
Sensitivity(y_pred = pred, y_true = mtcars$vs, positive = "1")
}
